@@ -119,69 +119,3 @@ pi__import_photos_cb (GthBrowser *browser,
 			 data,
 			 import_data_unref);
 }
-
-
-/* -- pi__dlg_preferences_construct_cb -- */
-
-
-#define PREFERENCES_DATA_KEY "photo-import-preference-data"
-
-
-typedef struct {
-	GtkBuilder *builder;
-	GSettings  *settings;
-} PreferencesData;
-
-
-static void
-preferences_data_free (PreferencesData *data)
-{
-	g_object_unref (data->settings);
-	g_object_unref (data->builder);
-	g_free (data);
-}
-
-
-static void
-adjust_orientation_checkbutton_toggled_cb (GtkToggleButton *button,
-					   PreferencesData *data)
-{
-	g_settings_set_boolean (data->settings, PREF_PHOTO_IMPORTER_ADJUST_ORIENTATION, gtk_toggle_button_get_active (button));
-}
-
-
-void
-pi__dlg_preferences_construct_cb (GtkWidget  *dialog,
-				  GthBrowser *browser,
-				  GtkBuilder *dialog_builder)
-{
-	PreferencesData *data;
-	GtkWidget       *general_vbox;
-	GtkWidget       *importer_options;
-	GtkWidget       *widget;
-
-	data = g_new0 (PreferencesData, 1);
-	data->builder = gtk_builder_new_from_resource ("/org/gnome/gThumb/photo_importer/data/ui/photo-importer-options.ui");
-	data->settings = g_settings_new (GTHUMB_PHOTO_IMPORTER_SCHEMA);
-
-	general_vbox = _gtk_builder_get_widget (dialog_builder, "general_page");
-	importer_options = _gtk_builder_get_widget (data->builder, "importer_options");
-	gtk_box_pack_start (GTK_BOX (general_vbox),
-			    importer_options,
-			    FALSE,
-			    FALSE,
-			    0);
-	/* move the options before the 'other' options */
-	gtk_box_reorder_child (GTK_BOX (general_vbox),
-			       importer_options,
-			       _gtk_container_get_n_children (GTK_CONTAINER (general_vbox)) - 2);
-
-	widget = _gtk_builder_get_widget (data->builder, "adjust_orientation_checkbutton");
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), g_settings_get_boolean (data->settings, PREF_PHOTO_IMPORTER_ADJUST_ORIENTATION));
-	g_signal_connect (widget,
-			  "toggled",
-			  G_CALLBACK (adjust_orientation_checkbutton_toggled_cb),
-			  data);
-
-	g_object_set_data_full (G_OBJECT (dialog), PREFERENCES_DATA_KEY, data, (GDestroyNotify) preferences_data_free);
-}
