@@ -31,6 +31,23 @@
 #define STRING_IS_VOID(x) (((x) == NULL) || (*(x) == 0))
 
 
+static GthTemplateCode Album_Header_Special_Codes[] = {
+	{ GTH_TEMPLATE_CODE_TYPE_SIMPLE, N_("Current page number"), 'p', 0 },
+	{ GTH_TEMPLATE_CODE_TYPE_SIMPLE, N_("Total number of pages"), 'P', 0 },
+	{ GTH_TEMPLATE_CODE_TYPE_SIMPLE, N_("Location"), 'L', 0 },
+	{ GTH_TEMPLATE_CODE_TYPE_DATE, N_("Current date"), 'D', 1 },
+};
+
+
+static GthTemplateCode Image_Header_Special_Codes[] = {
+	{ GTH_TEMPLATE_CODE_TYPE_SIMPLE, N_("Current image number"), 'i', 0 },
+	{ GTH_TEMPLATE_CODE_TYPE_SIMPLE, N_("Total number of images"), 'I', 0 },
+	{ GTH_TEMPLATE_CODE_TYPE_SIMPLE, N_("Original filename"), 'F', 0 },
+	{ GTH_TEMPLATE_CODE_TYPE_SIMPLE, N_("Comment"), 'C', 0 },
+	{ GTH_TEMPLATE_CODE_TYPE_DATE, N_("Current date"), 'D', 1 },
+};
+
+
 enum {
 	THEME_COLUMN_ID,
 	THEME_COLUMN_NAME,
@@ -43,13 +60,14 @@ enum {
 };
 
 typedef struct {
-	GthBrowser *browser;
-	GSettings  *settings;
-	GList      *file_list;
-	GtkBuilder *builder;
-	GtkWidget  *dialog;
-	GtkWidget  *thumbnail_caption_chooser;
-	GtkWidget  *image_attributes_chooser;
+	GthBrowser  *browser;
+	GthFileData *location;
+	GSettings   *settings;
+	GList       *file_list;
+	GtkBuilder  *builder;
+	GtkWidget   *dialog;
+	GtkWidget   *thumbnail_caption_chooser;
+	GtkWidget   *image_attributes_chooser;
 } DialogData;
 
 
@@ -60,6 +78,7 @@ destroy_cb (GtkWidget  *widget,
 	gth_browser_set_dialog (data->browser, "web_exporter", NULL);
 	_g_object_list_unref (data->file_list);
 	g_object_unref (data->settings);
+	g_object_unref (data->location);
 	g_object_unref (data->builder);
 	g_free (data);
 }
@@ -221,24 +240,156 @@ update_sensitivity (DialogData *data)
 }
 
 
-static void
-footer_entry_icon_press_cb (GtkEntry             *entry,
-			    GtkEntryIconPosition  icon_pos,
-			    GdkEvent             *event,
-			    gpointer              user_data)
+static gboolean
+text_preview_cb (TemplateFlags   flags,
+		 gunichar        parent_code,
+		 gunichar        code,
+		 char          **args,
+		 GString        *result,
+		 gpointer        user_data)
 {
 	DialogData *data = user_data;
-	GtkWidget  *help_box;
+	GDateTime  *timestamp;
+	char       *text;
 
-	if ((GTK_WIDGET (entry) == GET_WIDGET ("header_entry")) || (GTK_WIDGET (entry) == GET_WIDGET ("footer_entry")))
-		help_box = GET_WIDGET ("page_footer_help_table");
-	else
-		help_box = GET_WIDGET ("image_footer_help_table");
+	if (parent_code == 'D') {
+		/* strftime code, return the code itself. */
+		_g_string_append_template_code (result, code, args);
+		return FALSE;
+	}
 
-	if (gtk_widget_get_visible (help_box))
-		gtk_widget_hide (help_box);
-	else
-		gtk_widget_show (help_box);
+	if (code != 0)
+		g_string_append (result, "<span foreground=\"#4696f8\">");
+
+	switch (code) {
+	case 'p':
+		g_string_append (result, "1");
+		break;
+
+	case 'P':
+		g_string_append (result, "5");
+		break;
+
+	case 'D':
+		timestamp = g_date_time_new_now_local ();
+		text = g_date_time_format (timestamp,
+					   (args[0] != NULL) ? args[0] : DEFAULT_STRFTIME_FORMAT);
+		g_string_append (result, text);
+
+		g_free (text);
+		g_date_time_unref (timestamp);
+		break;
+
+	case 'i':
+		g_string_append (result, "1");
+		break;
+
+	case 'I':
+		g_string_append_printf (result, "%d", g_list_length (data->file_list));
+		break;
+
+	case 'F':
+		_g_string_append_markup_escaped (result, "%s", "filename.jpeg");
+		break;
+
+	case 'C':
+		_g_string_append_markup_escaped (result, "%s", _("Comment"));
+		break;
+
+	case 'L':
+		_g_string_append_markup_escaped (result, "%s", g_file_info_get_edit_name (data->location->info));
+		break;
+
+	default:
+		break;
+	}
+
+	if (code != 0)
+		g_string_append (result, "</span>");
+
+	return FALSE;
+}
+
+
+static void
+album_header_button_clicked_cb (DialogData *data,
+				const char *entry_name)
+{
+	GtkWidget *dialog;
+
+	dialog = gth_template_editor_dialog_new (Album_Header_Special_Codes,
+						 G_N_ELEMENTS (Album_Header_Special_Codes),
+						 0,
+						 _("Edit Template"),
+						 GTK_WINDOW (data->dialog));
+	gth_template_editor_dialog_set_preview_cb (GTH_TEMPLATE_EDITOR_DIALOG (dialog),
+						   text_preview_cb,
+						   data);
+	gth_template_editor_dialog_set_template (GTH_TEMPLATE_EDITOR_DIALOG (dialog),
+						 gtk_entry_get_text (GTK_ENTRY (GET_WIDGET (entry_name))));
+	g_signal_connect (dialog,
+			  "response",
+			  G_CALLBACK (gth_template_editor_dialog_default_response),
+			  GET_WIDGET (entry_name));
+	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+	gtk_window_present (GTK_WINDOW (dialog));
+}
+
+
+static void
+edit_header_button_clicked_cb (GtkButton *button,
+			       gpointer   user_data)
+{
+	album_header_button_clicked_cb ((DialogData *) user_data, "header_entry");
+}
+
+
+static void
+edit_footer_button_clicked_cb (GtkButton *button,
+			       gpointer   user_data)
+{
+	album_header_button_clicked_cb ((DialogData *) user_data, "footer_entry");
+}
+
+
+static void
+album_image_page_header_button_clicked_cb (DialogData *data,
+					   const char *entry_name)
+{
+	GtkWidget *dialog;
+
+	dialog = gth_template_editor_dialog_new (Image_Header_Special_Codes,
+						 G_N_ELEMENTS (Image_Header_Special_Codes),
+						 0,
+						 _("Edit Template"),
+						 GTK_WINDOW (data->dialog));
+	gth_template_editor_dialog_set_preview_cb (GTH_TEMPLATE_EDITOR_DIALOG (dialog),
+						   text_preview_cb,
+						   data);
+	gth_template_editor_dialog_set_template (GTH_TEMPLATE_EDITOR_DIALOG (dialog),
+						 gtk_entry_get_text (GTK_ENTRY (GET_WIDGET (entry_name))));
+	g_signal_connect (dialog,
+			  "response",
+			  G_CALLBACK (gth_template_editor_dialog_default_response),
+			  GET_WIDGET (entry_name));
+	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+	gtk_window_present (GTK_WINDOW (dialog));
+}
+
+
+static void
+edit_image_page_header_button_clicked_cb (GtkButton *button,
+					  gpointer   user_data)
+{
+	album_image_page_header_button_clicked_cb ((DialogData *) user_data, "image_page_header_entry");
+}
+
+
+static void
+edit_image_page_footer_button_clicked_cb (GtkButton *button,
+					  gpointer   user_data)
+{
+	album_image_page_header_button_clicked_cb ((DialogData *) user_data, "image_page_footer_entry");
 }
 
 
@@ -371,6 +522,7 @@ dlg_web_exporter (GthBrowser *browser,
 
 	data = g_new0 (DialogData, 1);
 	data->browser = browser;
+	data->location = gth_file_data_dup (gth_browser_get_location_data (browser));
 	data->file_list = _g_object_list_ref (file_list);
 	data->builder = _gtk_builder_new_from_file ("web-album-exporter.ui", "webalbums");
 	data->settings = g_settings_new (GTHUMB_WEBALBUMS_SCHEMA);
@@ -442,13 +594,18 @@ dlg_web_exporter (GthBrowser *browser,
 
 	g_free (default_sort_type);
 
-	gtk_entry_set_text (GTK_ENTRY (GET_WIDGET ("header_entry")),
-			    g_file_info_get_edit_name (gth_browser_get_location_data (browser)->info));
+	s_value = g_settings_get_string (data->settings, PREF_WEBALBUMS_HEADER);
+	gtk_entry_set_text (GTK_ENTRY (GET_WIDGET ("header_entry")), s_value);
+	g_free (s_value);
 
 	s_value = g_settings_get_string (data->settings, PREF_WEBALBUMS_FOOTER);
 	gtk_entry_set_text (GTK_ENTRY (GET_WIDGET ("footer_entry")), s_value);
 	g_free (s_value);
 
+	s_value = g_settings_get_string (data->settings, PREF_WEBALBUMS_IMAGE_PAGE_HEADER);
+	gtk_entry_set_text (GTK_ENTRY (GET_WIDGET ("image_page_header_entry")), s_value);
+	g_free (s_value);
+
 	s_value = g_settings_get_string (data->settings, PREF_WEBALBUMS_IMAGE_PAGE_FOOTER);
 	gtk_entry_set_text (GTK_ENTRY (GET_WIDGET ("image_page_footer_entry")), s_value);
 	g_free (s_value);
@@ -507,22 +664,6 @@ dlg_web_exporter (GthBrowser *browser,
 				  "clicked",
 				  G_CALLBACK (update_sensitivity),
 				  data);
-	g_signal_connect (GET_WIDGET ("header_entry"),
-			  "icon-press",
-			  G_CALLBACK (footer_entry_icon_press_cb),
-			  data);
-	g_signal_connect (GET_WIDGET ("footer_entry"),
-			  "icon-press",
-			  G_CALLBACK (footer_entry_icon_press_cb),
-			  data);
-	g_signal_connect (GET_WIDGET ("image_page_header_entry"),
-			  "icon-press",
-			  G_CALLBACK (footer_entry_icon_press_cb),
-			  data);
-	g_signal_connect (GET_WIDGET ("image_page_footer_entry"),
-			  "icon-press",
-			  G_CALLBACK (footer_entry_icon_press_cb),
-			  data);
 	g_signal_connect_swapped (GET_WIDGET ("single_index_checkbutton"),
 				  "toggled",
 				  G_CALLBACK (update_sensitivity),
@@ -539,6 +680,22 @@ dlg_web_exporter (GthBrowser *browser,
 				  "toggled",
 				  G_CALLBACK (update_sensitivity),
 				  data);
+	g_signal_connect (GET_WIDGET ("edit_header_button"),
+			  "clicked",
+			  G_CALLBACK (edit_header_button_clicked_cb),
+			  data);
+	g_signal_connect (GET_WIDGET ("edit_footer_button"),
+			  "clicked",
+			  G_CALLBACK (edit_footer_button_clicked_cb),
+			  data);
+	g_signal_connect (GET_WIDGET ("edit_image_page_header_button"),
+			  "clicked",
+			  G_CALLBACK (edit_image_page_header_button_clicked_cb),
+			  data);
+	g_signal_connect (GET_WIDGET ("edit_image_page_footer_button"),
+			  "clicked",
+			  G_CALLBACK (edit_image_page_footer_button_clicked_cb),
+			  data);
 
 	/* Run dialog. */
 
