@@ -58,9 +58,9 @@ typedef struct {
 
 
 static ExtensionCategory extension_category[] = {
-	{ EXTENSION_CATEGORY_ALL, N_("All"), "folder" },
-	{ EXTENSION_CATEGORY_ENABLED, N_("Enabled"), "folder" },
-	{ EXTENSION_CATEGORY_DISABLED, N_("Disabled"), "folder" },
+	{ EXTENSION_CATEGORY_ALL, NC_("Extensions", "All"), "folder" },
+	{ EXTENSION_CATEGORY_ENABLED, NC_("Extensions", "Enabled"), "folder" },
+	{ EXTENSION_CATEGORY_DISABLED, NC_("Extensions", "Disabled"), "folder" },
 	{ EXTENSION_CATEGORY_SEPARATOR, NULL, NULL },
 	{ "Browser", N_("Browser"), "folder" },
 	{ "Viewer", N_("Viewers"), "folder" },
@@ -488,7 +488,7 @@ create_extensions_row (GthExtensionDescription	*description,
 				  row_data);
 		gtk_box_pack_start (GTK_BOX (row_box), button, FALSE, FALSE, 0);
 	}
-	if ((extension != NULL) && (g_strcmp0 (description->authors[0], _("gthumb development team")) != 0)) {
+	if ((extension != NULL) && (g_strcmp0 (description->authors[0], _("gThumb Development Team")) != 0)) {
 		button = gtk_button_new_from_icon_name ("dialog-information-symbolic", GTK_ICON_SIZE_BUTTON);
 		gtk_widget_set_valign (button, GTK_ALIGN_CENTER);
 		g_signal_connect (button,
@@ -606,8 +606,6 @@ extensions__dlg_preferences_construct_cb (GtkWidget  *dialog,
 
 	for (scan = descriptions; scan; scan = scan->next) {
 		GthExtensionDescription *description = scan->data;
-		if (scan != descriptions)
-			_gtk_list_box_add_separator (GTK_LIST_BOX (data->extensions_list));
 		gtk_container_add (GTK_CONTAINER (data->extensions_list), create_extensions_row (description, data));
 	}
 
@@ -637,7 +635,7 @@ extensions__dlg_preferences_construct_cb (GtkWidget  *dialog,
 			cardinality = g_strdup_printf (CARDINALITY_FORMAT, get_category_cardinality (data, extension_category[i].id));
 			gtk_list_store_set (GTK_LIST_STORE (GET_WIDGET ("category_liststore")),
 					    &iter,
-					    CATEGORY_NAME_COLUMN, _(extension_category[i].name),
+					    CATEGORY_NAME_COLUMN, g_dpgettext2 (NULL, "Extensions", extension_category[i].name),
 					    CATEGORY_ID_COLUMN, extension_category[i].id,
 					    /* CATEGORY_ICON_COLUMN, extension_category[i].icon, */
 					    CATEGORY_SEPARATOR_COLUMN, FALSE,
