@@ -60,6 +60,7 @@ static const GActionEntry gth_browser_actions[] = {
 	{ "viewer-properties", toggle_action_activated, NULL, "false", gth_browser_activate_viewer_properties },
 	{ "unfullscreen", gth_browser_activate_unfullscreen },
 	{ "open-folder-in-new-window", gth_browser_activate_open_folder_in_new_window },
+	{ "show-progress-dialog", gth_browser_activate_show_progress_dialog },
 
 	{ "show-hidden-files", toggle_action_activated, NULL, "false", gth_browser_activate_show_hidden_files },
 	{ "sort-by", gth_browser_activate_sort_by },
@@ -81,6 +82,9 @@ static const GActionEntry gth_browser_actions[] = {
 
 	{ "file-list-select-all", gth_browser_activate_select_all },
 	{ "file-list-unselect-all", gth_browser_activate_unselect_all },
+
+	{ "set-filter", gth_browser_activate_set_filter, "s" },
+	{ "set-filter-all", gth_browser_activate_set_filter_all },
 };
 
 
@@ -127,6 +131,8 @@ static const GthShortcut gth_browser_shortcuts[] = {
 	{ "file-list-select-all", N_("Select all"), GTH_SHORTCUT_CONTEXT_BROWSER | GTH_SHORTCUT_CONTEXT_INTERNAL, GTH_SHORTCUT_CATEGORY_NAVIGATION, "<Primary>a" },
 	{ "file-list-unselect-all", N_("Select none"), GTH_SHORTCUT_CONTEXT_BROWSER | GTH_SHORTCUT_CONTEXT_INTERNAL, GTH_SHORTCUT_CATEGORY_NAVIGATION, "<Primary><Shift>a" },
 
+	{ "set-filter-all", N_("All"), GTH_SHORTCUT_CONTEXT_BROWSER, GTH_SHORTCUT_CATEGORY_FILTERS },
+
 	/* file list keybindings (read only) */
 
 	{ "file-list-move-cursor-right", NULL, GTH_SHORTCUT_CONTEXT_BROWSER | GTH_SHORTCUT_CONTEXT_INTERNAL, GTH_SHORTCUT_CATEGORY_HIDDEN, "Right" },
