@@ -31,17 +31,17 @@
 
 /* Properties */
 enum {
-        PROP_0,
-        PROP_ID,
-        PROP_ATTRIBUTES,
-        PROP_DISPLAY_NAME,
-        PROP_VISIBLE
+	PROP_0,
+	PROP_ID,
+	PROP_ATTRIBUTES,
+	PROP_DISPLAY_NAME,
+	PROP_VISIBLE
 };
 
 /* Signals */
 enum {
-        CHANGED,
-        LAST_SIGNAL
+	CHANGED,
+	LAST_SIGNAL
 };
 
 struct _GthTestPrivate {
@@ -56,6 +56,17 @@ static GthDuplicableInterface *gth_duplicable_parent_iface = NULL;
 static guint gth_test_signals[LAST_SIGNAL] = { 0 };
 
 
+static void gth_test_gth_duplicable_interface_init (GthDuplicableInterface *iface);
+
+
+G_DEFINE_TYPE_WITH_CODE (GthTest,
+			 gth_test,
+			 G_TYPE_OBJECT,
+			 G_ADD_PRIVATE (GthTest)
+			 G_IMPLEMENT_INTERFACE (GTH_TYPE_DUPLICABLE,
+						gth_test_gth_duplicable_interface_init))
+
+
 GQuark
 gth_test_error_quark (void)
 {
@@ -63,16 +74,6 @@ gth_test_error_quark (void)
 }
 
 
-static void gth_test_gth_duplicable_interface_init (GthDuplicableInterface *iface);
-
-
-G_DEFINE_TYPE_WITH_CODE (GthTest,
-		         gth_test,
-		         G_TYPE_OBJECT,
-		         G_IMPLEMENT_INTERFACE (GTH_TYPE_DUPLICABLE,
-		        		        gth_test_gth_duplicable_interface_init))
-
-
 static void
 gth_test_finalize (GObject *object)
 {
@@ -267,8 +268,6 @@ gth_test_class_init (GthTestClass *klass)
 {
 	GObjectClass *object_class;
 
-	g_type_class_add_private (klass, sizeof (GthTestPrivate));
-
 	object_class = (GObjectClass*) klass;
 	object_class->set_property = gth_test_set_property;
 	object_class->get_property = gth_test_get_property;
@@ -338,7 +337,7 @@ gth_test_gth_duplicable_interface_init (GthDuplicableInterface *iface)
 static void
 gth_test_init (GthTest *self)
 {
-	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GTH_TYPE_TEST, GthTestPrivate);
+	self->priv = gth_test_get_instance_private (self);
 	self->priv->id = g_strdup ("");
 	self->priv->attributes = g_strdup ("");
 	self->priv->display_name = g_strdup ("");
