@@ -89,6 +89,7 @@ gth_thumb_loader_finalize (GObject *object)
 	self = GTH_THUMB_LOADER (object);
 	_g_object_unref (self->priv->iloader);
 	_g_object_unref (self->priv->tloader);
+	_g_object_unref (self->priv->thumb_factory);
 
 	G_OBJECT_CLASS (gth_thumb_loader_parent_class)->finalize (object);
 }
@@ -238,7 +239,8 @@ static void
 gth_thumb_loader_construct (GthThumbLoader *self,
 			    int             requested_size)
 {
-	gth_thumb_loader_set_requested_size (self, requested_size);
+	if (requested_size > 0)
+		gth_thumb_loader_set_requested_size (self, requested_size);
 	self->priv->tloader = gth_image_loader_new (generate_thumbnail, self);
 	self->priv->iloader = gth_image_loader_new (load_cached_thumbnail, NULL);
 }
@@ -256,6 +258,27 @@ gth_thumb_loader_new (int requested_size)
 }
 
 
+GthThumbLoader *
+gth_thumb_loader_copy (GthThumbLoader *self)
+{
+	GthThumbLoader *loader;
+
+	loader = gth_thumb_loader_new (-1);
+
+	loader->priv->requested_size = self->priv->requested_size;
+	loader->priv->cache_max_size = self->priv->cache_max_size;
+	loader->priv->thumb_size = self->priv->thumb_size;
+	loader->priv->thumb_factory = _g_object_ref (self->priv->thumb_factory);
+
+	gth_thumb_loader_set_loader_func (loader, gth_image_loader_get_loader_func (self->priv->tloader));
+	gth_thumb_loader_set_use_cache (loader, self->priv->use_cache);
+	gth_thumb_loader_set_save_thumbnails (loader, self->priv->save_thumbnails);
+	gth_thumb_loader_set_max_file_size (loader, self->priv->max_file_size);
+
+	return loader;
+}
+
+
 void
 gth_thumb_loader_set_loader_func (GthThumbLoader     *self,
 				  GthImageLoaderFunc  loader_func)
@@ -716,7 +739,7 @@ watch_thumbnailer_cb (GPid     pid,
 		}
 
 		g_task_return_error (load_data->task, g_error_new_literal (G_IO_ERROR, G_IO_ERROR_CANCELLED, "script cancelled"));
-
+		load_data_unref (load_data);
 		return;
 	}
 
@@ -736,6 +759,43 @@ watch_thumbnailer_cb (GPid     pid,
 	}
 	else
 		failed_to_load_original_image (self, load_data);
+
+	load_data_unref (load_data);
+}
+
+
+static void
+load_with_system_thumbnailer (GthThumbLoader *self,
+			      LoadData       *load_data)
+{
+	char   *uri;
+	GError *error = NULL;
+
+	uri = g_file_get_uri (load_data->file_data->file);
+	if (gnome_desktop_thumbnail_factory_generate_from_script (self->priv->thumb_factory,
+								  uri,
+								  gth_file_data_get_mime_type (load_data->file_data),
+								  &load_data->thumbnailer_pid,
+								  &load_data->thumbnailer_tmpfile,
+								  &error))
+	{
+		load_data->thumbnailer_watch = g_child_watch_add (load_data->thumbnailer_pid,
+								  watch_thumbnailer_cb,
+								  load_data);
+		load_data->thumbnailer_timeout = g_timeout_add (MAX_THUMBNAILER_LIFETIME,
+								kill_thumbnailer_cb,
+								load_data);
+		load_data->cancellable_watch = g_timeout_add (CHECK_CANCELLABLE_DELAY,
+							      check_cancellable_cb,
+							      load_data);
+	}
+	else {
+		g_clear_error (&error);
+		failed_to_load_original_image (self, load_data);
+		load_data_unref (load_data);
+	}
+
+	g_free (uri);
 }
 
 
@@ -763,45 +823,24 @@ original_image_ready_cb (GObject      *source_object,
 		/* error loading the original image, try with the system
 		 * thumbnailer */
 
-		char *uri;
-
 		if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
 			g_task_return_error (load_data->task, error);
+			load_data_unref (load_data);
 			return;
 		}
 
 		g_clear_error (&error);
-
-		uri = g_file_get_uri (load_data->file_data->file);
-		if (gnome_desktop_thumbnail_factory_generate_from_script (self->priv->thumb_factory,
-									  uri,
-									  gth_file_data_get_mime_type (load_data->file_data),
-									  &load_data->thumbnailer_pid,
-									  &load_data->thumbnailer_tmpfile,
-									  &error))
-		{
-			load_data->thumbnailer_watch = g_child_watch_add (load_data->thumbnailer_pid,
-									  watch_thumbnailer_cb,
-									  load_data);
-			load_data->thumbnailer_timeout = g_timeout_add (MAX_THUMBNAILER_LIFETIME,
-									kill_thumbnailer_cb,
-									load_data);
-			load_data->cancellable_watch = g_timeout_add (CHECK_CANCELLABLE_DELAY,
-								      check_cancellable_cb,
-								      load_data);
-		}
-		else {
-			g_clear_error (&error);
-			failed_to_load_original_image (self, load_data);
-			load_data_unref (load_data);
-		}
-
-		g_free (uri);
-
+		load_with_system_thumbnailer (self, load_data);
 		return;
 	}
 
 	surface = gth_image_get_cairo_surface (image);
+	if (surface == NULL) {
+		g_object_unref (image);
+		load_with_system_thumbnailer (self, load_data);
+		return;
+	}
+
 	original_image_loaded_correctly (self,
 					 load_data,
 					 surface,
